/***************************************************************************
		Copyright (C) 2002-2008 Kentaro Kitagawa
		                   kitag@issp.u-tokyo.ac.jp
		
		This program is free software; you can redistribute it and/or
		modify it under the terms of the GNU Library General Public
		License as published by the Free Software Foundation; either
		version 2 of the License, or (at your option) any later version.
		
		You should have received a copy of the GNU Library General 
		Public License and a list of authors along with this program; 
		see the files COPYING and AUTHORS.
***************************************************************************/
//---------------------------------------------------------------------------
#include "testdriver.h"
#include "analyzer.h"
#include "xnodeconnector.h"
#include <qstatusbar.h>
#include "rand.h"

REGISTER_TYPE(XDriverList, TestDriver, "Test driver: random number generation");

XTestDriver::XTestDriver(const char *name, bool runtime, 
						 const shared_ptr<XScalarEntryList> &scalarentries,
						 const shared_ptr<XInterfaceList> &interfaces,
						 const shared_ptr<XThermometerList> &thermometers,
						 const shared_ptr<XDriverList> &drivers) :
    XDummyDriver<XPrimaryDriver>(name, runtime, scalarentries, interfaces, thermometers, drivers),
    m_entryX(create<XScalarEntry>("X", false, 
								  dynamic_pointer_cast<XDriver>(shared_from_this()), "%.3g")),
    m_entryY(create<XScalarEntry>("Y", false,
								  dynamic_pointer_cast<XDriver>(shared_from_this()), "%+.4f[K]"))
{
	scalarentries->insert(m_entryX);
	scalarentries->insert(m_entryY);
}

void
XTestDriver::showForms() {
//! impliment form->show() here
}
void
XTestDriver::start()
{
    m_thread.reset(new XThread<XTestDriver>(shared_from_this(), &XTestDriver::execute));
    m_thread->resume();
}
void
XTestDriver::stop()
{
    if(m_thread) m_thread->terminate();
//    m_thread->waitFor();
}
void
XTestDriver::analyzeRaw() throw (XRecordError&)
{
    //! Since raw buffer is Fast-in Fast-out, use the same sequence of push()es for pop()s
    m_x = pop<double>();
    m_y = pop<double>();
    m_entryX->value(m_x);
    m_entryY->value(m_y);
}
void
XTestDriver::visualize()
{
	//! impliment extra codes which do not need write-lock of record
	//! record is read-locked
}

void *
XTestDriver::execute(const atomic<bool> &terminated)
{
	while(!terminated)
	{
		msecsleep(10);
		double x = randMT19937() - 0.2;
		double y = randMT19937()- 0.2;
		clearRaw();
		push(x);
		push(y);
		finishWritingRaw(XTime::now(), XTime::now());
	}
	afterStop();
	return NULL;
}

